'' *****************************
'' *  Timer                    *
'' *  (C) 2006 Parallax, Inc.  *
'' *****************************


CON

'  TIX_DLY = 1_000_000 / 10_000                          ' for 10 ms tix period
  TIX_DLY = 1_000_000 / 1_000_000                                   ' for 1000 ms tix period
  MAX_STR = 16                                          ' maximum string length

  #0, TMR_SECS, TMR_MINS, TMR_HRS, TMR_DAYS

'--Para EEPROM----------------------------------------------------------------------------------
        eeprom        = %10100000          ' Device address for eeproms
        bootSCL       = 28                 ' Clock pin for boot eeprom
        bootSDA       = 29                 ' Data pin for boot eeprom
'-------------------------------------------------------------------------------------------------

OBJ

  boot : "i2cObject"                 ' I2C object for boot eeprom (requires special version)

VAR

  long  cogon, cog                                      ' cog status
  long  tmrStack[12]                                    ' local stack

  long  scs, mns, hrs, dys                         ' timer registers
  long  running                                         ' timer status
  byte  tmrStr[MAX_STR + 1]                             ' add one for zero terminator


PUB start : okay

'' Load background timer if cog available
'' -- timer is on hold and registers cleared

  stop
  reset
  okay := cogon := (cog := cognew(updateTimer, @tmrStack)) > 0


PUB stop

'' Unload timer object - frees a cog

  if cogon~                                             ' if object running, mark stopped
    cogstop(cog)                                        ' stop the cog


PUB set(h, m, s)

'' Put timer on hold and set hours, minutes, and seconds
'' -- tix and days registers are cleared

  hold                                                  ' put timer on hold
  scs := ||s                                            ' set registers (force positive)
  mns := ||m
  hrs := ||h


PUB reset

'' Put timer on hold and clear registers

  hold                                                  ' stop timer
  scs := mns := hrs := dys := 0                  ' reset registers


PUB hold

'' Put timer on hold without clearing registers

  running~                                              ' disable timer (set to false)


PUB run

'' Allow timer to run and update timer registers

  running~~                                             ' enable timer (set to true)
  boot.init(bootSDA,bootSCL)


PUB rdReg(reg)

'' Read a timer register

  case reg
    TMR_SECS : result := scs
    TMR_MINS : result := mns
    TMR_HRS  : result := hrs
    TMR_DAYS : result := dys


PUB wrReg(reg, value)

'' Write value to timer register
'' -- value is forced positive and truncated if required for register

  case reg
    TMR_SECS : scs := ||value // 60
    TMR_MINS : mns := ||value // 60
    TMR_HRS  : hrs := ||value // 24
    TMR_DAYS : dys := ||value 
  

PUB showTimer | s, m, h

  s := scs
  m := mns
  h := hrs
  
  bytefill(@tmrStr, 0, MAX_STR + 1)                     ' clear string   

  tmrStr[0] := (h / 10) + "0"                           ' convert 10's digit to ASCII char
  tmrStr[1] := (h // 10) + "0"                          ' convert 1's digit to ASCII char
  tmrStr[2] := ":"
  tmrStr[3] := (m / 10) + "0"
  tmrStr[4] := (m // 10) + "0"
  tmrStr[5] := ":"     
  tmrStr[6] := (s / 10) + "0"
  tmrStr[7] := (s // 10) + "0"

  return @tmrStr                                        ' return address of string
  
  
PUB SaveTimer | ps, pm, ph
          ps := scs
          pm := mns
          ph := hrs
          WriteBootLong (32748, ph)   '  Write values in eeprom
          waitcnt(clkfreq / 50 + cnt) ' 
          WriteBootLong (32752, pm) ' 
          waitcnt(clkfreq / 50 + cnt) ' 
          WriteBootLong (32756, ps) ' 
          waitcnt(clkfreq / 50 + cnt) ' 
         return false

PUB ReadTimer | ps, pm, ph
          ph:= ReadBootLong (32748) '  Read values from eeprom
          pm:= ReadBootLong (32752) '  
          ps:= ReadBootLong (32756) ' 

  bytefill(@tmrStr, 0, MAX_STR + 1)                     ' clear string   

  tmrStr[0] := (ph / 10) + "0"                           ' convert 10's digit to ASCII char
  tmrStr[1] := (ph // 10) + "0"                          ' convert 1's digit to ASCII char
  tmrStr[2] := ":"
  tmrStr[3] := (pm / 10) + "0"
  tmrStr[4] := (pm // 10) + "0"
  tmrStr[5] := ":"     
  tmrStr[6] := (ps / 10) + "0"
  tmrStr[7] := (ps // 10) + "0"

  return @tmrStr                                        ' return address of string

PRI updateTimer

'' Updates timer registers
'' -- start method launches this method into separate cog

  repeat                                                ' run until cog unloaded
    if running                                          ' if timer enabled
      waitcnt(clkfreq / TIX_DLY + cnt)                  '   do tix delay
        scs := ++scs // 60                              '     yes, update seconds
        if (scs == 0)                                   '     rollover?
          mns := ++mns // 60                            '       yes, update minutes

'          boot.WriteLocation(eeprom, 32760, 100, 16, 8)   IN EXAMPLE THIS WONT WORK FINE !!!! WHY ?? and how to autosave the time ??

          if (mns == 0)                                 '       rollover?
            hrs := ++hrs // 24                          '         yes, update hours
            if (hrs == 0)                               '         rollover? 
              ++dys                                     '           yes, increment days
              

PUB ReadBoot(location)
  return boot.readLocation(eeprom, location, 16, 8)
  
PUB ReadBootWord(location)
  return ReadBoot(location+1) << 8 | ReadBoot(location)

PUB ReadBootLong(location)
  return ReadBootWord(location+2) << 16 | ReadBootWord(location)

PUB WriteBoot(location, dato)
  waitcnt(clkfreq / 50 + cnt)
  return boot.WriteLocation(eeprom, location, dato, 16, 8)
  
PUB WriteBootWord(location,contents)
  WriteBoot(location+1,contents >> 8)
  WriteBoot(location,contents & $FF)

PUB WriteBootLong(location,contents)
  WriteBootWord(location+2,contents >> 16)
  WriteBootWord(location,contents & $FFFF)
             